# -*- coding: utf-8 -*-

import re

from collections import OrderedDict
from typing import List, Dict, Tuple

from PyQt5.QtCore import QDir, QCoreApplication

from libastra_csp.certificate import Certificate
from libastra_csp.certificate_indentity import CertificateIndentity
from libastra_csp.utils import Utils
from astra_csp_cryptopro.cprocsp_process import CprocspProcess
from astra_csp_cryptopro.cryptopro_settings import cryptopro_settings

translate = QCoreApplication.translate


class CryptoproCertificateIndentity(CertificateIndentity):
    def __init__(self, identity_string: str) -> None:
        super().__init__(identity_string=identity_string)
        self.__dict_re = re.compile(cryptopro_settings.dictionary['certificate_key_value_regexp'])
        self.__setup_dict()

    def __setup_dict(self) -> None:
        for match in self.__dict_re.finditer(self.identity_string):
            if match:
                key = match.group('key').strip()
                value = match.group('value').strip()
                if cryptopro_settings.dictionary.get('certificate_remove_quotes', False):
                    quotes_match = re.match(r'^"(?P<value>.*)"$', value)
                    if quotes_match:
                        value = quotes_match.group('value')
                    value = re.sub(r'""', '"', value)
                self[key] = value


class CertificateProcess(CprocspProcess):
    def __init__(self, bin_dir: QDir) -> None:
        super().__init__(bin_dir=bin_dir)
        self.__setup()
        self.__fields_re = re.compile(cryptopro_settings.dictionary['certificate_fields_regexp'], re.MULTILINE | re.U)

    def __setup(self) -> None:
        self.setProgram('bash')

        bin_name = self.bin_dir.filePath(cryptopro_settings.dictionary['get_certificates_bin_name'])
        bin_arguments = cryptopro_settings.dictionary.get('get_certificates_bin_arguments', [])

        first_env = cryptopro_settings.dictionary.get('get_certificates_process_environments', {})
        first_command_parts = [key + '=' + value for key, value in first_env.items()]

        second_env = cryptopro_settings.dictionary.get('get_certificates_process_environments_for_view', {})
        second_command_parts = [key + '=' + value for key, value in second_env.items()]

        self.setArguments(['-c', ' '.join(first_command_parts + [bin_name] + bin_arguments + ['&&'] +
                                          second_command_parts + [bin_name] + bin_arguments)])

    @staticmethod
    def get_command_parts(text: str) -> List[str]:
        return [match.group('certificates') for match
                in re.compile(cryptopro_settings.dictionary['certificates_compile_output_regexp'],
                              re.MULTILINE).finditer(text)]

    @staticmethod
    def get_certificates_context(parts: List[str]) -> Tuple[List[str], List[str]]:
        c_context = []
        localized_context = []
        for index, part in enumerate(parts):
            info = re.split(cryptopro_settings.dictionary['certificates_split_regexp'], part)
            info.remove('')

            if index == 0:
                c_context = info
            elif index == 1:
                localized_context = info
            else:
                break

        return c_context, localized_context

    @staticmethod
    def merge_fields(first_dictionary: Dict[str, str], second_dictionary: Dict[str, str]) -> Dict[str, str]:
        dictionary = {}

        values = list(second_dictionary.values())
        for index, key in enumerate(first_dictionary.keys()):
            try:
                dictionary[key] = values[index]
            except IndexError:
                dictionary[key] = first_dictionary.get(key)

        return dictionary

    def get_certificates(self) -> List[Certificate]:
        certificates = []  # type: List[Certificate]

        output = Utils.to_utf8(self.standard_output_buffer)
        command_parts = self.get_command_parts(output)
        c_context, localized_context = self.get_certificates_context(command_parts)

        cryptopro_certificate_fields = cryptopro_settings.dictionary['certificate_fields']
        for index, context in enumerate(c_context):
            fields = self.extract_fields(context)
            issuer_field = next(iter([field for field in fields
                                      if re.match(cryptopro_certificate_fields['Issuer'], field)]), None)
            subject_field = next(iter([field for field in fields
                                      if re.match(cryptopro_certificate_fields['Subject'], field)]), None)
            sha1_field = next(iter([field for field in fields
                                    if re.match(cryptopro_certificate_fields['SHA1'], field)]), None)
            if any(field is None for field in [issuer_field, subject_field, sha1_field]):
                continue

            try:
                view_output = localized_context[index]
                merged_fields = self.merge_fields(fields, self.extract_fields(view_output))
            except IndexError:
                view_output = context
                merged_fields = fields

            if fields[sha1_field] != merged_fields[sha1_field]:
                raise Exception(translate('text', 'Found different listing of certificates in different locales'))

            issuer_id = CryptoproCertificateIndentity(merged_fields.pop(issuer_field))
            certificate_id = CryptoproCertificateIndentity(merged_fields.pop(subject_field))
            sha1_id = merged_fields.pop(sha1_field)
            cert = Certificate(fingerprint=sha1_id,
                               subject=certificate_id,
                               view_output=view_output)
            cert.update(merged_fields)
            cert['Issuer'] = issuer_id
            cert['Subject'] = certificate_id
            cert['SHA1'] = sha1_id
            certificates.append(cert)
        return certificates

    def extract_fields(self, certificate: str) -> Dict[str, str]:
        fields = OrderedDict()  # type: Dict[str, str]
        for match in self.__fields_re.finditer(certificate):
            if match:
                key = match.group('key').strip()
                value = match.group('value').strip()
                fields[key] = value
        return fields

    def empty_list_error_occurred(self) -> bool:
        return self.output_error_code() == cryptopro_settings.dictionary['empty_list_error']
