# -*- coding: utf-8 -*-

from platform import machine
from typing import Tuple

from PyQt5.QtCore import QDir, QCoreApplication

from libastra_csp.common_functions import ResultStatus, remove_write_permissions
from libastra_csp.common_settings import CommonSettings
from libastra_csp.utils import Utils
from astra_csp_cryptopro.certificate_process import CertificateProcess
from astra_csp_cryptopro.create_file_signature_process import CreateFileSignatureProcess
from astra_csp_cryptopro.merge_certificates_process import MergeCertificatesProcess
from astra_csp_cryptopro.cryptopro_settings import cryptopro_settings
from astra_csp_cryptopro.verify_file_process import VerifyFileProcess
from astra_csp_cryptopro.cprocsp_process import CprocspProcess


translate = QCoreApplication.translate


def setup_cryptopro_bin_dir() -> QDir:
    arch = machine()
    if cryptopro_settings.bin_paths:
        paths = cryptopro_settings.bin_paths.get(arch)
    else:
        paths = None
    if not paths:
        raise Exception(translate('text', 'CryptoPro bin directory not set in settings for {arch} architecture. '
                                          'Please, check /etc/astra-csp-cryptopro/astra-csp-cryptopro.yaml')
                        .format(arch=arch))

    for path in paths:
        directory = QDir(path)
        if directory.exists():
            return directory
    else:
        raise Exception(translate('text', 'CryptoPro bin directory {paths} not exist. '
                                          'Please, check /etc/astra-csp-cryptopro/astra-csp-cryptopro.yaml')
                        .format(paths=', '.join(paths)))


def validate_system() -> None:
    try:
        current_max_level = Utils.get_mac_level()
    except Exception:
        raise Exception(translate('text', 'Error getting mandated level'))

    settings = CommonSettings()
    settings.load()
    available_max_level = settings.max_mac_level
    cryptopro_bin_dir = setup_cryptopro_bin_dir()
    if current_max_level > available_max_level:
        message = translate('text', 'The use of csp service at mandated '
                                    'level {current}(>{available}) is prohibited').format(current=current_max_level,
                                                                                          available=available_max_level)
        raise Exception(message)

    if not Utils.file_is_exists(cryptopro_bin_dir.filePath('cryptcp')):
        raise Exception(translate('text', 'Crypto provider not installed'))

    if not Utils.file_is_exists(cryptopro_bin_dir.filePath('certmgr')):
        raise Exception(translate('text', 'Certificate manager not installed'))


def get_status(process: CprocspProcess) -> ResultStatus:
    if process.timestamp_error_occurred():
        status = ResultStatus.Warning
    elif process.failed_output_error_code_occurred():
        status = ResultStatus.Error
    else:
        status = ResultStatus.Normal
    return status


def get_enriched_output(process: CprocspProcess) -> str:
    lines = process.output().split('\n')
    if process.timestamp_error_occurred():
        lines.append(translate('text', 'Error: Timestamp not received'))
    return '\n'.join(lines)


def remove_write_permissions_handler(process: CreateFileSignatureProcess) -> Tuple[ResultStatus, str]:
    status = get_status(process)
    output = get_enriched_output(process)
    if status != ResultStatus.Error:
        try:
            remove_write_permissions(process.document)
        except OSError as exception:
            status = ResultStatus.Warning
            lines = output.split('\n')
            lines.append(translate('text',
                                   'Warning: Failed to remove write permissions. {text}')
                         .format(text=exception.strerror))
            output = '\n'.join(lines)
    return status, output


def tuned_merge_certificate_process(callback=None, kill_time=0) -> MergeCertificatesProcess:
    merge_certificates_process = MergeCertificatesProcess(bin_dir=setup_cryptopro_bin_dir())
    merge_certificates_process.kill_time = kill_time
    if callback is not None:
        merge_certificates_process.completed.connect(callback)
    return merge_certificates_process


def tuned_certificates_process(callback=None, kill_time=0) -> CertificateProcess:
    get_certificate_process = CertificateProcess(bin_dir=setup_cryptopro_bin_dir())
    get_certificate_process.kill_time = kill_time
    if callback is not None:
        get_certificate_process.completed.connect(callback)
    return get_certificate_process


def tuned_verify_file_process(document: str, sig_path: str,
                              check_timestamp: bool, cli: bool = None, callback=None, kill_time=0) -> VerifyFileProcess:
    verify_process = VerifyFileProcess(bin_dir=setup_cryptopro_bin_dir())
    verify_process.setup_arguments(file_path=document,
                                   check_timestamp=check_timestamp,
                                   sig_path=sig_path,
                                   cli=cli)
    verify_process.kill_time = kill_time
    if callback is not None:
        verify_process.completed.connect(callback)
    return verify_process


def tuned_file_signature_process(document: str, sig_name: str, fingerprint: str,
                                 der_format: bool, time_address: str, password: str = None, cli: bool = False,
                                 callback=None, kill_time=0) -> CreateFileSignatureProcess:
    create_file_signature_process = CreateFileSignatureProcess(bin_dir=setup_cryptopro_bin_dir())
    create_file_signature_process.setup_arguments(file_path=document,
                                                  sig_name=sig_name,
                                                  fingerprint=fingerprint,
                                                  der_format=der_format,
                                                  time_address=time_address,
                                                  password=password,
                                                  cli=cli)
    create_file_signature_process.kill_time = kill_time
    if callback is not None:
        create_file_signature_process.completed.connect(callback)
    return create_file_signature_process


def validate_cryptopro_config() -> None:
    if cryptopro_settings.error:
        raise Exception(translate('text', 'Can not load cryptopro settings. {text}')
                        .format(text=cryptopro_settings.error))
    cryptopro_settings.validate()
