# -*- coding: utf-8 -*-
import pathlib
from argparse import ArgumentParser, RawTextHelpFormatter, Namespace

from PyQt5.QtCore import QCoreApplication

from libastra_csp.translator import Translator
from astra_csp_cryptopro.certificates_manipulation import print_certificates_list, merge_certificates
from astra_csp_cryptopro.sign_procedure import SignProcedure
from astra_csp_cryptopro.verify_procedure import VerifyProcedure

translate = QCoreApplication.translate


class CryptoPro:
    @staticmethod
    def load() -> None:
        Translator.load_all_translations()
        Translator.load_translation(directory='cryptopro', prefix='astra_csp_cryptopro_')

    @staticmethod
    def create_parser_base_group(parser: ArgumentParser):
        base_group = parser.add_mutually_exclusive_group(required=True)
        base_group.add_argument('--sign', action="store_true",
                                help=translate('text', 'Sign document. You must specify document path or name'))
        base_group.add_argument('--verify', action="store_true",
                                help=translate('text',
                                               'Verify document. You must specify document path only to verify attached '
                                               'signature or document and signature paths to verify it detached'))
        base_group.add_argument('--list', action="store_true",
                                help=translate('text', 'Get list of available certificates'))
        return base_group

    @staticmethod
    def setup_parser(parser: ArgumentParser) -> None:
        parser.formatter_class = RawTextHelpFormatter
        parser.description = translate('text', """\
Program is using for signing or verifying documents at CLI mode only.
Usage example:
    astra-csp-cryptopro --list (Get list of available certificates)
    astra-csp-cryptopro --verify --document <file> (Verify document with attached signature)
    astra-csp-cryptopro --verify --document <file> --signature <signature> (Verify document with detached signature)
    astra-csp-cryptopro --sign --document <file> -f=<fingerprint> --quiet (Sign document with selected \
certificate avoid messaging)
""")

        parser.add_argument('-d', '--document', type=str, nargs='?',
                            help=translate('text', translate('text', 'Document name or path to verify or sign')))
        parser.add_argument('-s', '--signature', type=str, nargs='?',
                            help=translate('text', 'Signature name or path to verify or sign'))
        parser.add_argument('-c', '--container', type=str, nargs='?',
                            help=translate('text', 'Container path to verify or sign document inside'))

        parser.add_argument('-q', '--quiet', action="store_true",
                            help=translate('text', 'Use this option if you don\'t want any output to stdout'))
        parser.add_argument('--merge', action="store_true",
                            help=translate('text', 'Merge certificates from removable drives'))
        parser.add_argument('-f', '--fingerprint', type=str,
                            help=translate('text', 'Certificate fingerprint for document singing (SHA1)'))
        parser.add_argument('-t', '--timestamp', action="store_true",
                            help=translate('text', 'Include timestamp for signed document or for verifying'))
        parser.add_argument('-e', '--encoding', type=str, choices=['DER', 'base64'],
                            help=translate('text',
                                           'Set encoding for singing. You can choose between <DER> and <base64>'))
        parser.add_argument('--create-container', action="store_true",
                            help=translate('text',
                                           'Create container with signature and signed document, after singing'))

        password_group = parser.add_mutually_exclusive_group()
        password_group.add_argument('-p', '--password', type=str, nargs='?',
                                    help=translate('text', 'Insert password for key container'))
        password_group.add_argument('--password-file', type=pathlib.Path, nargs='?',
                                    help=translate('text', 'Allow program get password for key container from file'))

    @staticmethod
    def exec(application: QCoreApplication, args: Namespace) -> None:
        if (args.sign or args.verify) and (not args.document and not args.container):
            raise Exception(translate('text', 'You should set the path or name of the document or container.'))

        if args.merge:
            merge_certificates()
        if args.list:
            print_certificates_list()
        elif args.sign:
            sign_procedure = SignProcedure(args)
            sign_procedure.setup()
            sign_procedure.execute()
        elif args.verify:
            verify_procedure = VerifyProcedure(args)
            verify_procedure.setup()
            verify_procedure.execute()