# -*- coding: utf-8 -*-

import argparse
import os
import sys
from typing import Optional

from PyQt5.QtCore import QCoreApplication

from libastra_csp.container import Container
from libastra_csp.application_cache import ApplicationCache
from libastra_csp.common_settings import CommonSettings
from libastra_csp.common_functions import check_result, clear_cache, emergency_exit, \
    new_container, check_paths, unpack_container, generate_signature_path
from libastra_csp.common_functions import print_not_quiet, parse_certificates
from astra_csp_cryptopro.certificates_manipulation import get_certificates_list
from astra_csp_cryptopro.cryptopro_settings import cryptopro_settings

from astra_csp_cryptopro.common_functions import validate_system, \
    tuned_file_signature_process, remove_write_permissions_handler, \
    ResultStatus, validate_cryptopro_config

translate = QCoreApplication.translate


class SignProcedure:
    def __init__(self, args: argparse.Namespace) -> None:
        self.__file_path = None  # type: Optional[str]
        self.__container_path = None  # type: Optional[str]
        self.__sig_path = None  # type: Optional[str]
        self.__container = None  # type: Optional[Container]

        self.__document = args.document
        self.__signature = args.signature
        self.__container_name = args.container
        self.__fingerprint = args.fingerprint
        self.__encoding = args.encoding
        self.__timestamp = args.timestamp
        self.__create_container = args.create_container
        self.__quiet = args.quiet
        self.__password = args.password
        self.__password_file = args.password_file

        self.__cache = ApplicationCache()
        self.__common_settings = CommonSettings()

    def setup(self) -> None:
        try:
            validate_system()
            validate_cryptopro_config()
            self.__common_settings.load()
            self.__set_paths()
            self.__check_for_container()
            self.__check_signature_path()
            self.__fingerprint_check()
            self.__check_password_file()
        except Exception as exception:
            print(*exception.args, file=sys.stderr)
            emergency_exit(self.__cache)

    def __set_paths(self) -> None:
        self.__file_path, self.__sig_path, self.__container_path = \
            check_paths(self.__document, self.__signature, self.__container_name)

    def __check_for_container(self) -> None:
        if self.__container_name:
            if self.__create_container:
                raise Exception(translate('text', 'You have got container, already. '
                                                  'Remove --create-container option to update existed container '
                                                  'or choose another file.'))
            else:
                print_not_quiet(translate('text', 'Preparing container...'),
                                file=sys.stdout, quiet=self.__quiet)
                self.__container, possible_file_path = unpack_container(self.__container_path, self.__cache)
                self.__file_path = possible_file_path if self.__document is None \
                    else "{dir}/{file}".format(dir=self.__container.mount_dir.path(), file=self.__document)

                if not self.__file_path:
                    raise Exception(translate('text', 'Can not recognize document name.'))

                if self.__signature:
                    self.__sig_path = "{dir}/{file}".format(dir=os.path.dirname(self.__file_path),
                                                            file=self.__signature)

    def __check_signature_path(self) -> None:
        if not self.__sig_path:
            self.__sig_path = generate_signature_path(self.__file_path, '.p7s', self.__timestamp)

    def __fingerprint_check(self) -> None:
        try:
            certificates = parse_certificates(get_certificates_list())
            if not certificates:
                raise Exception(translate('text', 'Certificates not available. Empty certificate list.'))

            if self.__fingerprint:
                if self.__fingerprint not in certificates.keys():
                    raise Exception(translate('text', 'Current fingerprint does not exist. '
                                                      'Check the list of available certificates.'))
            else:
                self.__fingerprint = list(certificates)[0]
                print_not_quiet(translate('text', 'You did not specify fingerprint. Document will be signed with:'),
                                certificates.get(self.__fingerprint), file=sys.stdout, quiet=self.__quiet)
        except IndexError:
            raise Exception(translate('text', 'Can not get any certificate automatically. Check your vault, please.'))

    def __check_password_file(self):
        if self.__password_file:
            try:
                with self.__password_file.open(mode='r', encoding='UTF-8') as file:
                    self.__password = file.readline().strip()
            except OSError:
                raise Exception(translate('text', 'File with password does not exist.'))
            except UnicodeError:
                raise Exception(translate('text', 'Can not decode password string from {path}.'
                                          .format(path=self.__password_file.resolve())))

    def execute(self) -> None:
        der_format = True if self.__encoding == 'DER' else False
        if self.__timestamp:
            time_address = cryptopro_settings.time_address
        else:
            time_address = None

        file_signature_process = tuned_file_signature_process(
            document=self.__file_path,
            sig_name=self.__sig_path,
            fingerprint=self.__fingerprint,
            der_format=der_format,
            time_address=time_address,
            password=self.__password,
            cli=True)

        print_not_quiet(translate('text', 'Start singing...'), file=sys.stdout, quiet=self.__quiet)
        file_signature_process.start()

        if not self.__password:
            print_not_quiet(translate('text', 'You did not set password for the key container. '
                                              'If it is present indicate it explicitly, please.'),
                            file=sys.stdout, quiet=self.__quiet)

        file_signature_process.waitForFinished(-1)
        try:
            check_result(file_signature_process)
            status, output = remove_write_permissions_handler(file_signature_process)
            if status == ResultStatus.Error:
                raise Exception(translate('text', 'Process failed.'), status)

            elif self.__create_container:
                self.__cache.create_dir()
                self.__container = new_container(file_signature_process.document, self.__cache.cache_path())
                self.__container.save([file_signature_process.document,
                                       file_signature_process.signature])
                try:
                    os.remove(self.__file_path)
                    os.remove(self.__sig_path)
                except OSError as exception:
                    raise Exception(translate('text', 'Container error, failed to remove original files.'),
                                    *exception.args)
            elif self.__container:
                try:
                    self.__container.save([file_signature_process.document,
                                           file_signature_process.signature], True)
                except Exception as exception:
                    raise Exception(translate('text', 'Container error, failed to update container.'), *exception.args)
        except Exception as exception:
            print(*exception.args, file=sys.stderr)
            emergency_exit(self.__cache)
        else:
            clear_cache(self.__cache)
            try:
                self.__cache.cache_path()
                basename = os.path.basename(file_signature_process.signature)
                signature_path_text = translate('text',
                                                'Signature created inside container. Signature name is: ') + basename
                signature_path_text += '\n'
                signature_path_text += translate('text', 'Container path is: ') + self.__container.path
            except AttributeError:
                signature_path_text = translate('text', 'Signature path: ') + file_signature_process.signature
            print_not_quiet(output, signature_path_text, file=sys.stdout, quiet=self.__quiet)
