# -*- coding: utf-8 -*-

import argparse
import os
import sys
from typing import Optional

from PyQt5.QtCore import QCoreApplication

from libastra_csp.container import Container
from libastra_csp.application_cache import ApplicationCache
from libastra_csp.common_functions import print_not_quiet, unpack_container, check_result, \
    clear_cache, emergency_exit, check_paths
from astra_csp_cryptopro.common_functions import tuned_verify_file_process, validate_cryptopro_config

translate = QCoreApplication.translate


class VerifyProcedure:
    def __init__(self, args: argparse.Namespace) -> None:
        self.__file_path = None  # type: Optional[str]
        self.__sig_path = None  # type: Optional[str]
        self.__container_path = None  # type: Optional[str]
        self.__container = None  # type: Optional[Container]

        self.__document = args.document
        self.__signature = args.signature
        self.__container_name = args.container
        self.__timestamp = args.timestamp
        self.__quiet = args.quiet

        self.__cache = ApplicationCache()

    def setup(self) -> None:
        try:
            validate_cryptopro_config()
            self.__set_paths()
            self.__check_for_container()
        except Exception as exception:
            print(*exception.args, file=sys.stderr)
            emergency_exit(self.__cache)

    def __set_paths(self) -> None:
        self.__file_path, self.__sig_path, self.__container_path = \
            check_paths(self.__document, self.__signature, self.__container_name)

    def __check_for_container(self) -> None:
        if self.__signature and self.__container_path:
            print_not_quiet(translate('text', 'Preparing container...'),
                            file=sys.stdout, quiet=self.__quiet)
            self.__container, self.__file_path = unpack_container(self.__container_path, self.__cache)
            if self.__document:
                self.__file_path = "{dir}/{file}".format(dir=self.__container.mount_dir.path(),
                                                         file=self.__document)
            if not self.__file_path:
                raise Exception(translate('text', 'Can not get document path. Specify document name, please.'))

            self.__sig_path = "{dir}/{file}".format(dir=os.path.dirname(self.__file_path),
                                                    file=self.__signature)

    def execute(self) -> None:
        verify_process = tuned_verify_file_process(
            document=self.__file_path,
            sig_path=self.__sig_path,
            check_timestamp=self.__timestamp,
            cli=True)
        print_not_quiet(translate('text', 'Start verifying...'), file=sys.stdout, quiet=self.__quiet)
        verify_process.start()
        verify_process.waitForFinished(-1)
        try:
            check_result(verify_process)
        except Exception as exception:
            print(*exception.args, file=sys.stderr)
            emergency_exit(self.__cache)
        else:
            clear_cache(self.__cache)
            print_not_quiet(verify_process.output(), file=sys.stdout, quiet=self.__quiet)
